@@ -1,95 +1,4 @@
-//! Feature: completion with imports-on-the-fly
-//!
-//! When completing names in the current scope, proposes additional imports from other modules or crates,
-//! if they can be qualified in the scope, and their name contains all symbols from the completion input.
-//!
-//! To be considered applicable, the name must contain all input symbols in the given order, not necessarily adjacent.
-//! If any input symbol is not lowercased, the name must contain all symbols in exact case; otherwise the containing is checked case-insensitively.
-//!
-//! ```
-//! fn main() {
-//!     pda$0
-//! }
-//! # pub mod std { pub mod marker { pub struct PhantomData { } } }
-//! ```
-//! ->
-//! ```
-//! use std::marker::PhantomData;
-//!
-//! fn main() {
-//!     PhantomData
-//! }
-//! # pub mod std { pub mod marker { pub struct PhantomData { } } }
-//! ```
-//!
-//! Also completes associated items, that require trait imports.
-//! If any unresolved and/or partially-qualified path precedes the input, it will be taken into account.
-//! Currently, only the imports with their import path ending with the whole qualifier will be proposed
-//! (no fuzzy matching for qualifier).
-//!
-//! ```
-//! mod foo {
-//!     pub mod bar {
-//!         pub struct Item;
-//!
-//!         impl Item {
-//!             pub const TEST_ASSOC: usize = 3;
-//!         }
-//!     }
-//! }
-//!
-//! fn main() {
-//!     bar::Item::TEST_A$0
-//! }
-//! ```
-//! ->
-//! ```
-//! use foo::bar;
-//!
-//! mod foo {
-//!     pub mod bar {
-//!         pub struct Item;
-//!
-//!         impl Item {
-//!             pub const TEST_ASSOC: usize = 3;
-//!         }
-//!     }
-//! }
-//!
-//! fn main() {
-//!     bar::Item::TEST_ASSOC
-//! }
-//! ```
-//!
-//! NOTE: currently, if an assoc item comes from a trait that's not currently imported, and it also has an unresolved and/or partially-qualified path,
-//! no imports will be proposed.
-//!
-//! .Fuzzy search details
-//!
-//! To avoid an excessive amount of the results returned, completion input is checked for inclusion in the names only
-//! (i.e. in `HashMap` in the `std::collections::HashMap` path).
-//! For the same reasons, avoids searching for any path imports for inputs with their length less than 2 symbols
-//! (but shows all associated items for any input length).
-//!
-//! .Import configuration
-//!
-//! It is possible to configure how use-trees are merged with the `importMergeBehavior` setting.
-//! Mimics the corresponding behavior of the `Auto Import` feature.
-//!
-//! .LSP and performance implications
-//!
-//! The feature is enabled only if the LSP client supports LSP protocol version 3.16+ and reports the `additionalTextEdits`
-//! (case-sensitive) resolve client capability in its client capabilities.
-//! This way the server is able to defer the costly computations, doing them for a selected completion item only.
-//! For clients with no such support, all edits have to be calculated on the completion request, including the fuzzy search completion ones,
-//! which might be slow ergo the feature is automatically disabled.
-//!
-//! .Feature toggle
-//!
-//! The feature can be forcefully turned off in the settings with the `rust-analyzer.completion.autoimport.enable` flag.
-//! Note that having this flag set to `true` does not guarantee that the feature is enabled: your client needs to have the corresponding
-//! capability enabled.
-
+//! See [`import_on_the_fly`].
 use ide_db::helpers::{
     import_assets::{ImportAssets, ImportCandidate},
     insert_use::ImportScope,
@@ -105,6 +14,97 @@ use crate::{
 
 use super::Completions;
 
+// Feature: Completion With Autoimport
+//
+// When completing names in the current scope, proposes additional imports from other modules or crates,
+// if they can be qualified in the scope, and their name contains all symbols from the completion input.
+//
+// To be considered applicable, the name must contain all input symbols in the given order, not necessarily adjacent.
+// If any input symbol is not lowercased, the name must contain all symbols in exact case; otherwise the containing is checked case-insensitively.
+//
+// ```
+// fn main() {
+//     pda$0
+// }
+// # pub mod std { pub mod marker { pub struct PhantomData { } } }
+// ```
+// ->
+// ```
+// use std::marker::PhantomData;
+//
+// fn main() {
+//     PhantomData
+// }
+// # pub mod std { pub mod marker { pub struct PhantomData { } } }
+// ```
+//
+// Also completes associated items, that require trait imports.
+// If any unresolved and/or partially-qualified path precedes the input, it will be taken into account.
+// Currently, only the imports with their import path ending with the whole qualifier will be proposed
+// (no fuzzy matching for qualifier).
+//
+// ```
+// mod foo {
+//     pub mod bar {
+//         pub struct Item;
+//
+//         impl Item {
+//             pub const TEST_ASSOC: usize = 3;
+//         }
+//     }
+// }
+//
+// fn main() {
+//     bar::Item::TEST_A$0
+// }
+// ```
+// ->
+// ```
+// use foo::bar;
+//
+// mod foo {
+//     pub mod bar {
+//         pub struct Item;
+//
+//         impl Item {
+//             pub const TEST_ASSOC: usize = 3;
+//         }
+//     }
+// }
+//
+// fn main() {
+//     bar::Item::TEST_ASSOC
+// }
+// ```
+//
+// NOTE: currently, if an assoc item comes from a trait that's not currently imported, and it also has an unresolved and/or partially-qualified path,
+// no imports will be proposed.
+//
+// .Fuzzy search details
+//
+// To avoid an excessive amount of the results returned, completion input is checked for inclusion in the names only
+// (i.e. in `HashMap` in the `std::collections::HashMap` path).
+// For the same reasons, avoids searching for any path imports for inputs with their length less than 2 symbols
+// (but shows all associated items for any input length).
+//
+// .Import configuration
+//
+// It is possible to configure how use-trees are merged with the `importMergeBehavior` setting.
+// Mimics the corresponding behavior of the `Auto Import` feature.
+//
+// .LSP and performance implications
+//
+// The feature is enabled only if the LSP client supports LSP protocol version 3.16+ and reports the `additionalTextEdits`
+// (case-sensitive) resolve client capability in its client capabilities.
+// This way the server is able to defer the costly computations, doing them for a selected completion item only.
+// For clients with no such support, all edits have to be calculated on the completion request, including the fuzzy search completion ones,
+// which might be slow ergo the feature is automatically disabled.
+//
+// .Feature toggle
+//
+// The feature can be forcefully turned off in the settings with the `rust-analyzer.completion.autoimport.enable` flag.
+// Note that having this flag set to `true` does not guarantee that the feature is enabled: your client needs to have the corresponding
+// capability enabled.
 pub(crate) fn import_on_the_fly(acc: &mut Completions, ctx: &CompletionContext) -> Option<()> {
     if !ctx.config.enable_imports_on_the_fly {
         return None;
@@ -163,11 +163,11 @@ pub(crate) fn position_for_import<'a>(
     import_candidate: Option<&ImportCandidate>,
 ) -> Option<&'a SyntaxNode> {
     Some(match import_candidate {
-        Some(ImportCandidate::Path(_)) => ctx.name_ref_syntax.as_ref()?.syntax(),
+        Some(ImportCandidate::Path(_)) => ctx.name_syntax.as_ref()?.syntax(),
         Some(ImportCandidate::TraitAssocItem(_)) => ctx.path_qual()?.syntax(),
         Some(ImportCandidate::TraitMethod(_)) => ctx.dot_receiver()?.syntax(),
         None => ctx
-            .name_ref_syntax
+            .name_syntax
             .as_ref()
             .map(|name_ref| name_ref.syntax())
             .or_else(|| ctx.path_qual().map(|path| path.syntax()))
@@ -180,23 +180,18 @@ fn import_assets(ctx: &CompletionContext, fuzzy_name: String) -> Option<ImportAs
     if let Some(dot_receiver) = ctx.dot_receiver() {
         ImportAssets::for_fuzzy_method_call(
             current_module,
-            ctx.sema.type_of_expr(dot_receiver)?,
+            ctx.sema.type_of_expr(dot_receiver)?.original,
             fuzzy_name,
             dot_receiver.syntax().clone(),
         )
     } else {
         let fuzzy_name_length = fuzzy_name.len();
-        let approximate_node = match current_module.definition_source(ctx.db).value {
-            hir::ModuleSource::SourceFile(s) => s.syntax().clone(),
-            hir::ModuleSource::Module(m) => m.syntax().clone(),
-            hir::ModuleSource::BlockExpr(b) => b.syntax().clone(),
-        };
         let assets_for_path = ImportAssets::for_fuzzy_path(
             current_module,
             ctx.path_qual().cloned(),
             fuzzy_name,
             &ctx.sema,
-            approximate_node,
+            ctx.token.parent()?,
         )?;
 
         if matches!(assets_for_path.import_candidate(), ImportCandidate::Path(_))
@@ -701,8 +696,8 @@ fn main() {
 }
 "#,
             expect![[r#"
-                ct SPECIAL_CONST (use dep::test_mod::TestTrait) DEPRECATED
                 fn weird_function() (use dep::test_mod::TestTrait) fn() DEPRECATED
+                ct SPECIAL_CONST (use dep::test_mod::TestTrait) DEPRECATED
             "#]],
         );
     }
@@ -1189,4 +1184,40 @@ impl<T> Private for T {}
             expect![[r#""#]],
         );
     }
+
+    #[test]
+    fn regression_9760() {
+        check(
+            r#"
+struct Struct;
+fn main() {}
+
+mod mud {
+    fn func() {
+        let struct_instance = Stru$0
+    }
+}
+"#,
+            expect![[r#"
+                st Struct (use crate::Struct)
+            "#]],
+        );
+    }
+
+    #[test]
+    fn flyimport_pattern() {
+        check(
+            r#"
+mod module {
+    pub struct Struct;
+}
+fn function() {
+    let Str$0
+}
+"#,
+            expect![[r#"
+                st Struct (use module::Struct)
+            "#]],
+        );
+    }
 }
